/*
 *! @brief          EventMode-Callbacks fuer C#-Beispiel
 *
 *! @author         Markus Biggel
 * letzte Aenderung 14.07.2010
 *
 *! @addtogroup csharpusage Verwendung in C#
 *! @file Beispiele/UsageInC-Sharp/src/KPCallBack.cs
 *! @ingroup csharpusage csharp csharpusageev
 */
using System;
using System.Runtime.InteropServices;
using KeyPilotCom;


namespace keypilot
{
class KPCallBack
{
    protected static KPCallBack     m_instance = new KPCallBack();

    //! Delegates, die an die native DLL uebergeben werden
    protected KeyPilot.KpNoticeDelegate kpNoticeDelegate;
    protected KeyPilot.KpNoticeDisconnectDelegate kpDisconnectDelegate;

    public delegate void KpEventDelegate(
                             int deviceID_,
                             KP_KEYDATA kpData_,
                             bool disconnected_ );
    //! C# Eventmethode, die im Falle eines Events aus den Callback-
    //! Methoden #kpNoticeDelegate und #kpDisconnectDelegate heraus aufgerufen
    //! wird (hier nur der Vereinfachung halber zusammen in einer
    //! Methode zusammengefuehrt; lassen sich natuerlich auch trennen).
    protected event KpEventDelegate m_kpEventMethod = null;

    private bool _eventModeStarted = false;
    //! Property, das angibt ob der Event Modus gestartet ist oder nicht
    public bool EventModeStarted {
        get {
            return _eventModeStarted;
        }

        protected set {
            _eventModeStarted = value;
        }
    }


    //! Singleton-Klasse; deswegen keine Instanzierung ausserhalb m&ouml;glich
    protected KPCallBack()
    {
        // beide Delegates MUESSEN auf Klassenebene erstellt werden, damit
        // der Garbage-Collector diese nicht "aufraeumt", obwohl sie noch
        // gebraucht werden (die Verwendung als Callback in einer nativen
        // DLL "zaehlt" fuer den GC nicht als "gebrauchen")
        kpNoticeDelegate = new KeyPilot.KpNoticeDelegate(
                                            kpNoticeCallback );
        kpDisconnectDelegate = new KeyPilot.KpNoticeDisconnectDelegate(
                                            kpDisconnectCallback );
    }


    public static KPCallBack getInstance()
    {
        return m_instance;
    }


    public void raiseEvent(         int deviceID_,
                                    KP_KEYDATA kpData_,
                                    bool disconnected_ )
    {
        // Aufruf der extern eingehaengten Methode(n):
        if ( EventModeStarted && m_kpEventMethod != null ) {
            m_kpEventMethod( deviceID_, kpData_, disconnected_ );
        }
    }


    // *******************************************************************
    //! @brief  Startet den Event Modus.
    //!
    //! Schaltet den Event Modus ein und ruft bei eintreffenden Events
    //!   die Methoden kpNoticeCallback bzw. kpDisconnectCallback auf.
    //!
    //! @param[in] deviceID_    Ger&auml;te-ID, f&uuml;r die der Event
    //!                           Modus eingeschaltet wird.
    //! @param[in] evMethod_    externe Methode, die aufgerufen werden
    //!                           soll bei eintreffenden Events (sowohl
    //!                           "Notice als auch "Disconnect").
    // *******************************************************************
    public void startEventMode(     int deviceID_,
                                    KpEventDelegate evMethod_ )
    {
        KeyPilot.kpUSBSetEventmode( deviceID_,
                                    true,   // EvMode einschalten
                                    true,   // unendliches Warten
                                    kpNoticeDelegate,
                                    kpDisconnectDelegate );

        if ( evMethod_ != null ) {
            m_kpEventMethod += evMethod_;
        }

        EventModeStarted = true;
    }


    // *******************************************************************
    //! @brief  Stoppt den Event Modus.
    //!
    //! Schaltet den Event Modus aus, d.h. es werden keine Events mehr
    //!   an die Methoden kpNoticeCallback bzw. kpDisconnectCallback
    //!   weitergeleitet.
    //!
    //! @param[in] deviceID_    Ger&auml;te-ID, f&uuml;r die der Event
    //!                           Modus eingeschaltet wird.
    //! @param[in] evMethod_    externe Methode, die ausgehaengt werden
    //!                           soll.
    // *******************************************************************
    public void stopEventMode(      int deviceID_,
                                    KpEventDelegate evMethod_ )
    {
        EventModeStarted = false;

        KeyPilot.kpUSBSetEventmode( deviceID_,
                                    false,  // EvMode ausschalten
                                    false,
                                    null,
                                    null );

        if ( evMethod_ != null ) {
            try {
                m_kpEventMethod -= evMethod_;
            } catch {
            }
        }
    }


    // *******************************************************************
    //! @brief  KeyPilot-Notice Callback.
    //!
    //! Wird bei eingeschaltetem Event Modus aufgerufen, wenn 
    //!   "Notice"-Events eintreffen.
    //!
    //! @param[in] deviceID_    Ger&auml;te-ID, f&uuml;r die der Event
    //!                           abgesetzt wird.
    //! @param[in] kpData_      Daten des Events.
    // *******************************************************************
    public static void kpNoticeCallback(
                                    int deviceID_,
                                    ref KP_KEYDATA kpData_ )
    {
        if ( String.IsNullOrEmpty(kpData_.command) && !kpData_.validData ) {
            // leerer (unnuetzer?) Event
            // --> ignorieren
            return;
        }

        KPCallBack kpInst = KPCallBack.getInstance();
        kpInst.raiseEvent( deviceID_, kpData_, false );

        return;
    }


    // *******************************************************************
    //! @brief  KeyPilot-DisconnectNotice Callback.
    //!
    //! Wird bei eingeschaltetem Event Modus aufgerufen, wenn 
    //!   "DisconnectNotice"-Events eintreffen.
    //!
    //! @param[in] deviceID_    Ger&auml;te-ID, f&uuml;r die der Event
    //!                           abgesetzt wird.
    // *******************************************************************
    public static void kpDisconnectCallback(
                                    int deviceID_ )
    {
        KPCallBack kpInst = KPCallBack.getInstance();
        kpInst.raiseEvent( deviceID_, new KP_KEYDATA(), true );

        return;
    }
} // END_CLASS KPCallBack
} // END namespace keypilot