﻿/*
 *! @brief          Einstellungsdialog fuer C#-Beispiel
 *
 *! @author         Markus Biggel
 * letzte Aenderung 30.09.2010
 *
 *! @addtogroup csharpusage Verwendung in C#
 *! @file Beispiele/UsageInC-Sharp/src/kpDialog.cs
 *! @ingroup csharpusage csharp csharpusageev
*/
using System;
using System.Windows.Forms;
using KeyPilotCom;


namespace keypilot
{
    /// <summary>
    /// Description of kpDialog.
    /// </summary>
    public partial class kpDialog : Form
    {
        private int _deviceID = -1;
        // *******************************************************************
        //! @brief Property zur Abfrage der verbundenen Ger&auml;te-ID
        // *******************************************************************
        public int DeviceID {
            get {
                return _deviceID;
            }

            protected set {
                _deviceID = value;
            }
        }


        // *******************************************************************
        //! @brief  Konstruktor
        //!
        //! Initialisiert die graphischen Komponenten und Klassenvariablen.
        // *******************************************************************
        public kpDialog()
        {
            InitializeComponent();

            DeviceID = -1;
        }


        // *******************************************************************
        //! @brief  Methode scannt beim Aufruf nach angeschlossenen KeyPilots.
        //!
        //! Scannt nach KeyPilot-Ger&auml;ten und f&uuml;llt die Combobox
        //!   @ref cbKpList.
        //!
        //! @return Anzahl der gefundenen KeyPilot-Ger&auml;ten.
        // *******************************************************************
        private int ScanKeyPilots()
        {
            // Combobox erstmal leeren:
            cbKpList.Items.Clear();

            // nach KeyPilot-Ger&auml;ten scannen
            KeyPilot.kpUSBScanDevices();

            // Anzahl der gefundenen Ger&auml;te abfragen
            int numKeyPilots = KeyPilot.kpUSBGetNumberOfDevices();
            numKP.Text = numKeyPilots.ToString();
            if ( numKeyPilots <= 0 )
            {
                if ( DeviceID >= 0 ) {
                    // wir sind (scheinbar) noch mit einem Ger&auml;t verbunden
                    // -> trennen
                    btnDisconnect_Click( this, null );
                }

                // keine KeyPilots gefunden
                // -> fertig
                return -1;
            }

            // ausreichend Speicher f&uuml;r die Daten der KeyPilot-Ger&auml;te
            // holen
            KP_USB_DEV[] kpDevs;
            kpDevs = new KP_USB_DEV[ numKeyPilots ];

            // Abfrage der Ger&auml;tedaten
            KP_RETURN devLstRet;
            devLstRet = KeyPilot.kpUSBGetDeviceList( kpDevs, numKeyPilots );
            if ( devLstRet != KP_RETURN.KP_OK ) {
                if ( DeviceID >= 0 ) {
                    // wir sind (scheinbar) noch mit einem Ger&auml;t verbunden
                    // -> trennen
                    btnDisconnect_Click( this, null );
                }

                return -1;
            }

            // Ger&auml;te-IDs in der GUI zur Auswahl stellen
            int idxSelected = -1;
            for ( int i = 0; i < numKeyPilots; i++ ) {
                int idx = cbKpList.Items.Add( kpDevs[i] );

                if ( DeviceID >= 0 && kpDevs[i].deviceID == DeviceID ) {
                    idxSelected = idx;
                }
            }
            // das bereits selektierte Ger&auml;t wieder selektieren:
            if ( idxSelected >= 0 ) {
                try {
                    cbKpList.SelectedIndex = idxSelected;
                } catch ( ArgumentOutOfRangeException ) {
                }
            } else {
                if ( DeviceID >= 0 ) {
                    // wir sind (scheinbar) noch mit einem Ger&auml;t verbunden,
                    // das es nicht mehr gibt
                    // -> trennen
                    btnDisconnect_Click( this, null );
                }

                // erstes gefundenes Ger&auml;t selektieren:
                cbKpList.SelectedIndex = 0;

            }

            return numKeyPilots;
        }


        // *******************************************************************
        //! @brief  EventHandler-Methode, die bei Show/Hide aufgerufen wird.
        //!
        //! Dieser EventHandler scannt jedesmal nach einem Show nach
        //!   KeyPilot-Ger&auml;ten und f&uuml;llt die Combobox @ref cbKpList
        //!   mit den IDs.
        // *******************************************************************
        void EvVisibilityChanged(   object sender_,
                                    EventArgs evArgs_ )
        {
            if ( Visible ) {
                ScanKeyPilots();
            }
        }


        // *******************************************************************
        //! @brief  Verbindungsaufbau zum ausgew&auml;hlten KeyPilot-Ger&auml;t
        //
        //! @param[in]  sender_         wird nicht verwendet.
        //! @param[in]  evArgs_         wird nicht verwendet.
        // *******************************************************************
        void btnConnect_Click(      object sender_,
                                    EventArgs evArgs_ )
        {
            // pruefen, ob in der Combobox ein Element selektiert ist.
            if ( cbKpList.SelectedIndex < 0 ) {
                return;
            }

            KP_USB_DEV kpDev = (KP_USB_DEV)cbKpList.SelectedItem;

            DeviceID = kpDev.deviceID;
            if ( DeviceID < 0 ) {
                return;
            }

            connectedKP.Text = "-";

            // Mit dem KeyPilot-Ger&auml;t verbinden
            KP_RETURN ret;
            ret = KeyPilot.kpUSBConnectToDevice( DeviceID );
            if ( ret == KP_RETURN.KP_OK ) {
                // Ger&auml;tedaten des verbundenen KeyPilot-Ger&auml;t holen
                KP_USB_DEV kpDevCon;
                kpDevCon = new KP_USB_DEV();
                KeyPilot.kpUSBGetDevice( DeviceID, ref kpDevCon );

                connectedKP.Text = kpDevCon.ToString();;

                btnConnect.Enabled = false;
                btnDisconnect.Enabled = true;
            }
        }


        // *******************************************************************
        //! @brief  Dialog verstecken.
        //
        //! @param[in]  sender_         wird nicht verwendet.
        //! @param[in]  evArgs_         wird nicht verwendet.
        // *******************************************************************
        void btnClose_Click(        object sender_,
                                    EventArgs evArgs_ )
        {
            Hide();
        }


        // *******************************************************************
        //! @brief  Verbindungsabbau des ausgew&auml;hlten KeyPilot-Ger&auml;t
        //
        //! @param[in]  sender_         wird nicht verwendet.
        //! @param[in]  evArgs_         wird nicht verwendet.
        // *******************************************************************
        void btnDisconnect_Click(   object sender_,
                                    EventArgs evArgs_ )
        {
            if ( DeviceID >= 0 ) {
                // Ger&auml;teverbindung wieder trennen
                KeyPilot.kpUSBDisconnectFromDevice( DeviceID );

                DeviceID = -1;
                btnConnect.Enabled = true;
                btnDisconnect.Enabled = false;
            }

            connectedKP.Text = "-";
        }
    }
}