/**
 * \file KEYPILOT_USB_DEVICE.java
 * 
 * @author AFahrenkrog
 *
 */


/**
 * \class KEYPILOT_USB_DEVICE
 * 
 * This class implements in Java the KeyPilotUSBUserDLL.h functions
 * by calling a wrapper library KeyPilotUSBWrapper.dll together with
 * KeyPilotUSB.dll
 * 
 * The functions with the 'native' keyword access the DLL
 * 
 */
public class KEYPILOT_USB_DEVICE {
	
	/**
	 * \name Connection status
	 * @{
	 */
	public static final int NOT_AVAILABLE = 0;
	public static final int AVAILABLE = 1;
	public static final int CONNECTED = 2;
	/**
	 * @}
	 */
	
	/**
	 * \name LED Colors
	 * @{
	 */
	public static final int KP_LED_COLOR_GREEN = 0;
	public static final int KP_LED_COLOR_RED = 1;
	public static final int KP_LED_COLOR_OFF = 2;
	/**
	 * @}
	 */
	
	/**
	 * \name DLL String length
	 * @{
	 */
	public static final int DLL_VERSION_LENGTH = 10;
	public static final int KP_FIRMWARE_VERSION_LENGTH = 11;
	public static final int KP_SERIAL_NUMER_LENGTH = 50;
	public static final int KP_ERRSTRING_LENGTH = 100;
	/**
	 * @}
	 */
	
	/**
	 * \name Error codes
	 * @{
	 */
	public static final int KP_OK                        	= 0;
	public static final int KP_ERR_NO_DEVICE_FOUND      	= -1;
	public static final int KP_ERR_CONNECTION_FAILED    	= -4;
	public static final int KP_ERR_NOT_CONNECTED        	= -5;
	public static final int KP_ERR_WRONG_PAGE_SIZE      	= -6;
	public static final int KP_ERR_BUFFER_TOO_SMALL     	= -7;
	public static final int KP_ERR_DEVICE_BUSY          	= -10;
	public static final int KP_ERR_DEVICE_NOT_FOUND     	= -11;
	public static final int KP_ERR_UNKNOWN              	= -12;
	public static final int KP_ERR_ILLEGAL_DEVICE       	= -13;
	public static final int KP_ERR_DATASIZE_WRONG       	= -14;
	public static final int KP_ERR_READ_ERROR           	= -15;
	public static final int KP_ERR_TIMEOUT              	= -16;
	public static final int KP_ERR_WRONG_NR_OF_PAGES    	= -17;
	public static final int KP_ERR_WRONG_KEY_TYPE       	= -18;
	public static final int KP_ERR_IN_EVENTMODE         	= -19;
	public static final int KP_ERR_ILLEGAL_PAGE         	= -20;
	public static final int KP_ERR_WRITE_ERROR          	= -21;
	public static final int KP_ERR_CORRUPT_DATA         	= -23;
	public static final int KP_ERR_NOT_IMPLEMENTED      	= -24;
	public static final int KP_ERR_NO_KEY_ON_PILOT      	= -25;
	public static final int KP_ERR_TO_FEW_DATA          	= -26;
	public static final int KP_ERR_INDEX_OUT_OF_RANGE   	= -27;
	/**
	 * @}
	 */
	
	/**
	 * \var static String libraryVersion
	 * \brief String holding the DLL Version
	 */
	private static String libraryVersion;
	
	/** 
	 * \name USB KeyPilot Device Data
	 * @{
	 */
	private String	serialNumber;
	private int		connectionStatus;
	private int		eventMode;
	private String	vendorName;
	private int		vendorID;
	private int		productID;
	private int		version;
	private String	friendlyName;
	private String	description;
	private String	prName;
	private int		deviceID;
	/**
	 * @}
	 */
	
	/**
	 * \var KEYPILOT_KEYDATA keyData
	 * \brief Variable holding the present key data
	 */
	private KEYPILOT_KEYDATA keyData;
	
	/**
	 * \var String errorString
	 * \brief Stores the error string returned by kpUSBGetErrorString
	 */
	private String errorString;
	
	/**
	 * \var String firmwareversion
	 * \brief Stores the string returned by kpUSBGetFirmwareVersion
	 */
	private String firmwareVersion;
	
	/**
	 * \brief Gets the serial number string
	 * @return The serial number
	 */
	public String getSerialNumber(){
		return serialNumber;
	}
	
	/**
	 * \brief Gets the vendor name string
	 * @return The vendor name string
	 */
	public String getVendorName(){
		return vendorName;
	}
	
	/**
	 * \brief Gets the friendly name string
	 * @return The friendly name string
	 */
	public String getFriendlyName(){
		return friendlyName;
	}
	
	/**
	 * \brief Gets the description string
	 * @return The description string
	 */
	public String getDescription(){
		return description;
	}
	
	/**
	 * \brief Gets the product name string
	 * @return The product name string
	 */
	public String getPrName(){
		return prName;
	}
	
	/**
	 * \brief Gets the connection status
	 * @return NOT_AVAILABLE, AVAILABLE or CONNECTED
	 */
	public int getConnectionStatus(){
		return connectionStatus;
	}
	
	/**
	 * \brief Gets the event mode
	 * @return 0 or 1
	 */
	public int getEventMode(){
		return eventMode;
	}
	
	/**
	 * \brief Gets the device ID
	 * @return Integer assigned to the device
	 */
	public int getDeviceID(){
		return deviceID;
	}
	
	/**
	 * \brief Gets the vendor ID
	 * @return Integer with the vendor ID
	 */
	public int getVendorID(){
		return vendorID;
	}
	
	/**
	 * \brief Gets the production ID
	 * @return Integer with the production ID
	 */
	public int getProductID(){
		return productID;
	}
	
	/**
	 * \brief Gets the firmware version
	 * @return The firmware version number
	 */
	public int getVersion(){
		return version;
	}
	
	/**
	 * \brief Gets the data of the key on the pilot
	 * @return KEYPILOT_KEYDATA object with the actual data
	 */
	public KEYPILOT_KEYDATA getKeyData(){
		return keyData;
	}
	
	/**
	 * \brief Gets the error string
	 * @return String containing the error message
	 */
	public String getErrorString(){
		return errorString;
	}
	
	/**
	 * \brief Gets the firmware version
	 * @return String containing the firmware version
	 */
	public String getFirmwareVersion(){
		return firmwareVersion;
	}
	
	/**
	 * \brief Gets the library verson
	 * @return String containing the DLL library version
	 */
	public String getLibraryVersion(){
		return libraryVersion;
	}
	
	/**
	 * \name Native DLL Functions
	 * \details These functions are derived from the 
	 * DLL file. The functions are accessed through the
	 * KeyPilotUSBWrapper.dll, which in turn call the 
	 * corresponding KeyPilotUSB.dll functions
	 * @{
	 */
	public static native int kpUSBScanDevices();
	public static native int kpUSBGetNumberOfDevices();
	public static native int kpUSBConnectToDevice(int deviceID);
	public static native int kpUSBDisconnectFromDevice(int deviceID);
	public static native int kpUSBGetDeviceListIndex(KEYPILOT_USB_DEVICE javaKp, int index);
	public static native int kpUSBGetDevice(int deviceID, KEYPILOT_USB_DEVICE javaKp);
	public static native int kpUSBGetKeyID(int deviceID, KEYPILOT_KEYDATA kpdata);
	public static native int kpUSBSetLED(int deviceID, int color);
	public static native int kpUSBDeactivateTimeout(int deviceID);
	public static native int kpUSBGetKeyPageSize(int deviceID);
	public static native int kpUSBGetNrOfPages(int deviceID);
	public static native int kpUSBGetLastError();
	public static native int kpUSBSetSettingmode(int deviceID, int active);
	public static native int kpUSBDoesDeviceExist(int deviceID);
	
	public static native int kpUSBReadPageFromKey(int deviceID, int pageNr, byte[] dataBuffer, int bufferSize);
	public static native int kpUSBWritePagesToKey(int deviceID, int nrOfPages, byte[] data, int dataSize);
	
	public static native int kpUSBGetErrorString(int errorCode, KEYPILOT_USB_DEVICE javaKp);
	public static native int kpUSBGetSerialNumber(int deviceID, KEYPILOT_USB_DEVICE javaKp);
	public static native int kpUSBGetFirmwareVersion(int deviceID, KEYPILOT_USB_DEVICE javaKp);
	public static native int kpUSBGetLibraryVersion();
	/**
	 * @}
	 */
}
